

#    experiment run control for AIDA - taken from sys.tcl

global MIDAS_DataAcquisitionControlServer_Verbose
set MIDAS_DataAcquisitionControlServer_Verbose 0


variable MaxASICs 4
variable MaxChans 16

variable Options


proc Dump_Raw_Data {} {return [Dump_Raw_Aida_Data]}

proc Set_Options {} {
   variable Options

   load_Options

   if {![info exists Options(Aida_GroupBase)]} {set Options(Aida_GroupBase) 1}
   if {![info exists Options(Include.Aida)]} {set Options(Include.Aida) 1}

   if {![info exists Options(Rate.channels)]} {set Options(Rate.channels) 150}
   if {![info exists Options(Stat.channels)]} {set Options(Stat.channels) 150}
   if {![info exists Options(Stat.shift)]} {set Options(Stat.shift) 6}
   if {![info exists Options(Stat.offset)]} {set Options(Stat.offset) 64}

   if {![info exists Options(Aida_Hist_L_Enable)]} {set Options(Aida_Hist_L_Enable) 1}
   if {![info exists Options(Aida_Hist_H_Enable)]} {set Options(Aida_Hist_H_Enable) 1}
   if {![info exists Options(Aida_Hist_D_Enable)]} {set Options(Aida_Hist_D_Enable) 1}

   if {![info exists Options(Aida.channels)]} {set Options(Aida.channels) 65536}
   if {![info exists Options(Aida.offset)]} {set Options(Aida.offset) 0}
   if {![info exists Options(Aida.shift)]} {set Options(Aida.shift) 0}

   if {![info exists Options(ASIC.settings)]} {set Options(ASIC.settings) undefined}

   if {![info exists Options(ExtClk)]} {set Options(ExtClk) 1}

   update_Options

}


proc Dump_Raw_Aida_Data {} {

   set info "ASIC data $::DataAcquisition::ACQSERVER\n"

   for {set offset 0} {$offset <= 300} {incr offset 100} {

     DataAcqCommand 19 "Getting Raw Data" "17 $offset" 2
     set data [ReadNetVar NetVar.EXEC.Response]

     foreach word $data {
         set word $word
         if {$word == 0xaa5555aa} then {
             append info "\n$word - end of dump\n"
             break
         }
         if {$word == 0x55aaaa55} then {
             append info "\n$word - end of event\n"
             continue
         }
         append info "$word "
         set w [expr ($word >> 30) & 3]
         set det [expr ($word >> 27) & 7]
         set chan [expr ($word >> 20) & 31]
         if {$chan < 16} {set range L} else {set range H; set chan [expr $chan & 15]}
         set dat [format 0x%4x [expr $word & 0x0000ffff]]

         append info "Word $w: ASIC $det: channel $chan: range $range: "

         switch $w {
            0 {append info "ADC data $dat"}
            1 {append info "Time 47-32 $dat"}
            2 {append info "Time 31-16 $dat"}
            3 {append info "Time 15-0 $dat"}
         }
         append info "\n"
     }
   }

   return "$info"
}


proc create_Aida_spectra {} {
   variable MaxASICs
   variable MaxChans 
   variable Options

   set Me $::DataAcquisition::ACQSERVER

     puts "Creating spectra for $Me"

#   for the data channels

   for {set Det 1} {$Det <= $MaxASICs} {incr Det} {

      puts "Creating spectra for ASIC $Det"

      for {set Ch 0} {$Ch < $MaxChans} {incr Ch} {

        if {$Options(Aida_Hist_L_Enable) == 1} {

        set z [catch {CreateSpectrum1D $Det.$Ch.L $Options(Aida.channels)} m]

#    Allow "Spectrum already exists" so that further spectra can be created
         if {$z!=0 && $m==0x30004} then {set z 0}
         if {$z!=0} then {
            puts "Attempting to create spectrum $Det.$Ch.L"
            error $m
         }

        set z [catch {SpectrumPutTitle $Det.$Ch.L [set Me]_$Det.$Ch.L} m]

         }

        if {$Options(Aida_Hist_H_Enable) == 1} {

        set z [catch {CreateSpectrum1D $Det.$Ch.H $Options(Aida.channels)} m]

#    Allow "Spectrum already exists" so that further spectra can be created
         if {$z!=0 && $m==0x30004} then {set z 0}
         if {$z!=0} then {
            puts "Attempting to create spectrum $Det.$Ch.H"
            error $m
         }

        set z [catch {SpectrumPutTitle $Det.$Ch.H [set Me]_$Det.$Ch.H} m]

         }
     }
   }

#  for the discriminator data (hit) and its rate 

        if {$Options(Aida_Hist_D_Enable) == 1} {

        set z [catch {CreateSpectrum1D Hit 64} m]

#    Allow "Spectrum already exists" so that further spectra can be created
         if {$z!=0 && $m==0x30004} then {set z 0}
         if {$z!=0} then {
            puts "Attempting to create spectrum Hit"
            error $m
         }

        set z [catch {SpectrumPutTitle Hit [set Me]_Hit} m]

        set z [catch {CreateSpectrum1D HitRate 64} m]

#    Allow "Spectrum already exists" so that further spectra can be created
         if {$z!=0 && $m==0x30004} then {set z 0}
         if {$z!=0} then {
            puts "Attempting to create spectrum HitRate"
            error $m
         }

        set z [catch {SpectrumPutTitle HitRate [set Me]_HitRate} m]

        }
}

proc map_Aida_spectra {} {
   variable MaxASICs 
   variable MaxChans
   variable Options
   upvar #0 SpectraHandle f

   set Me $::DataAcquisition::ACQSERVER

   set SpectraDefs [file join /tmp SpectraDefs.dat]

      set z [catch {open $SpectraDefs w} f]
      if {$z != 0} then {
          puts "Cannot open file $SpectraDefs $f"
          return
      }

      map_spectrum "Stat 0 0 0 $Options(Stat.shift)" 

#   for the data channels

   for {set Det 1} {$Det <= $MaxASICs} {incr Det} {

     for {set Ch 0} {$Ch < $MaxChans} {incr Ch} {

       if {$Options(Aida_Hist_L_Enable) == 1} {

       set group [expr $Options(Aida_GroupBase)+(($Det-1)*32)+$Ch]
       set item 0
       map_spectrum "$Det.$Ch.L $group $item $Options(Aida.offset) $Options(Aida.shift)"
       }

       if {$Options(Aida_Hist_H_Enable) ==1} {

       set group [expr $Options(Aida_GroupBase)+(($Det-1)*32)+16+$Ch]
       set item 0
       map_spectrum "$Det.$Ch.H $group $item $Options(Aida.offset) $Options(Aida.shift)"
       }
     }
   }

   if {$Options(Aida_Hist_D_Enable) == 1} {
        map_spectrum "Hit [expr $Options(Aida_GroupBase)+128] 0 0 0"
   }

   close $f
}

proc doSetUp {} {
    variable Options

puts "executing doSetUp"

     set_Options; make-UIregisters

     DataAcqCommand 1 "Halt"

     unmap_spectra

     delete_spectra

      set Me $::DataAcquisition::ACQSERVER

      CreateSpectrum1D Stat $Options(Stat.channels)
      set z [catch {SpectrumPutTitle Stat [set Me]_Stat} m]

      CreateSpectrum1D Rate $Options(Rate.channels)
      set z [catch {SpectrumPutTitle Rate [set Me]_Rate} m]

    create_Aida_spectra

    map_Aida_spectra 

    map_spectra

    Setup_Electronics

   ParameterRestore

     DataAcqCommand 3 "Setup"

    WriteNetVar NetVar.DACQ.PollTicks 100
    WriteNetVar NetVar.DACQ.Crdoutenable 1   ;# enable data readout

    puts "completed doSetUp"

}

   proc ParameterRestore {} {
      variable Options

      set ::AIDA::DBNodeBase $Options(ASIC.settings)
      set ::AIDA::DBNodeName $::DataAcquisition::ACQSERVER
      
      puts "Restoring FEE64 settings from $::AIDA::DBRootName/$::AIDA::DBNodeBase for $::DataAcquisition::ACQSERVER"

      for {set asic 1} {$asic < 5} {incr asic} {
         for {set i 0} {$i < 27} {incr i} {
            set vl [DBGetValue $asic.$i]
            WriteAIDARegister [expr 0x4000 + (($asic-1)*64) + $i]] $vl
         }
      }

      puts "FEE64 settings restored from $::AIDA::DBRootName/$::AIDA::DBNodeBase for $::DataAcquisition::ACQSERVER"

      for {set asic 1} {$asic <= 4} {incr asic} {
         WriteAIDARegister [expr 0x4000 + (($asic-1)*64) + 28] 0   ;#  load control registers
      }

#       ::AIDA::WriteRegister HasBeenRestored 1

#        Elog INFORMATION "FEE64 settings restored from $::AIDA::DBRootName/$::AIDA::DBNodeBase for $::DataAcquisition::ACQSERVER"

   }

   proc ParameterSave {} {
      variable Options

      set ::AIDA::DBNodeBase $Options(ASIC.settings)
      set ::AIDA::DBNodeName $::DataAcquisition::ACQSERVER
      
      puts "Saving FEE64 settings to $::AIDA::DBRootName/$::AIDA::DBNodeBase for $::DataAcquisition::ACQSERVER"

      for {set asic 1} {$asic < 5} {incr asic} {
         for {set i 0} {$i < 27} {incr i} {
            set vl [ReadAIDARegister [expr 0x4000 + (($asic-1)*64) + $i]]
            DBSetValue $asic.$i [format "0x%02x" [expr $vl & 255]]
         }
      }

      puts "FEE64 settings saved to $::AIDA::DBRootName/$::AIDA::DBNodeBase for $::DataAcquisition::ACQSERVER"

#       ::AIDA::WriteRegister HasBeenRestored 1

#        Elog INFORMATION "FEE64 settings saved to $::AIDA::DBRootName/$::AIDA::DBNodeBase for $::DataAcquisition::ACQSERVER"

   }

   proc SpecSave {} {
      global env
      variable Options

      set ::AIDA::DBNodeBase Histograms
      set ::AIDA::DBNodeName $::DataAcquisition::ACQSERVER
      
      puts "Saving FEE64 histograms for $::DataAcquisition::ACQSERVER"

      set f1 [file join $env(MIDASBASE) SpecData $::DataAcquisition::ACQSERVER]
      set z [catch {file mkdir $f1} m]
      set f2 [file join $env(MIDASBASE) SpecData $::DataAcquisition::ACQSERVER [date +%Y%h%d-%H.%M.%S]]
      set z [catch {file mkdir $f2} m]

      set SpecNames [SpectrumNames]
      foreach item $SpecNames {
         if {[first $item] != 1} {continue}   ;#   not a value histogram !!
         set SpecName [second $item]
         set SpecTitle [SpectrumGetTitle $SpecName]
         set SpecDetails [SpectrumDetails $SpecName]

         set Dim  [first $SpecDetails]
         set Base [first [second $SpecDetails]]
         set Range [first [third $SpecDetails]]

         if {$Dim != 1} {continue}   ;#   not a 1D histogram

         set SpecData [SpectrumRead1D $SpecName $Base $Range]

         set z [catch {open [file join $f2 $SpecName] w} f]
         foreach item $SpecData {puts $f $item}
         close $f
      }

      puts "FEE64 histograms saved to $f2 for $::DataAcquisition::ACQSERVER"


#        Elog INFORMATION "FEE64 histograms saved to $::AIDA::DBRootName/$::AIDA::DBNodeBase for $::DataAcquisition::ACQSERVER"

   }

   proc ExtCLK {} {

          WriteAIDARegister [expr 0x000 + 5] 0x20    ;#  reset  DCM
          WriteAIDARegister [expr 0x000 + 5] 0x1b    ;#  select clock from HDMI 
          set status [ReadAIDARegister  [expr 0x000 + 1] ]        ;#  status
          if {[expr $status] != 7} {puts "Clock not locked. status = $status"}
   }

   proc IntCLK {} {

          WriteAIDARegister [expr 0x000 + 5] 0x20    ;#  reset  DCM
          WriteAIDARegister [expr 0x000 + 5] 0x7    ;#  select internal clock
          set status [ReadAIDARegister  [expr 0x000 + 1] ]        ;#  status
          if {[expr $status] != 7} {puts "Clock not locked. status = $status"}
   }


proc Reset_Electronics {} {    ;#   used by RESET

     WriteAIDARegister 0x400 0   	 ;#   disable Timestamp counter
     WriteAIDARegister 0x300 0x4000   ;#    unset enable readout and reset 

     WriteAIDARegister 0x600 0        ;#  clear DMA
     WriteAIDARegister 0x600 0x8000   ;#  set reset DMA
     WriteAIDARegister 0x600 0        ;#  and clear
}

proc Setup_Electronics {} {    ;#   used by SETUP
    variable Options


    set Me $::DataAcquisition::ACQSERVER
    set TSMaster $::AIDA::TSMaster

    WriteAIDARegister 0x000 3

    if {$Options(ExtClk) == 0} {IntCLK} else {ExtCLK}

    if {$Me == $TSMaster} {    
	WriteAIDARegister 7 3 ;# set MBS clock for loopback and enable MBS
    } else {
	WriteAIDARegister 7 0 ;# set MBS signals as inputs
    }

    WriteAIDARegister 0x315 0x180    ;#  MBS triggger delay
    WriteAIDARegister 0x307 0xf      ;# enable readout for all four of the ASICs

     WriteAIDARegister 8 0 ;# initial setup to enable SYNC return path

     WriteAIDARegister 0x400 0   ;#   disable Timestamp counter
     WriteAIDARegister 0x410 0   ;#   disable Master Timestamp counter
     WriteAIDARegister 0x416 0   ;#   set Timestamp high 32 bits
     WriteAIDARegister 0x415 0   ;#   set Timestamp low 32 bits and load timestamp counter
     WriteAIDARegister 0x400 1   ;#   enable Timestamp counter

     if {$Me == $TSMaster} { 
	WriteAIDARegister 0x410 1   ;#   enable Master timestamp
     }

     WriteAIDARegister 0x603 0x07000000  ;# set DMA MEM_Base
     WriteAIDARegister 0x604 0x000fffff  ;# set DMA HWM
     WriteAIDARegister 0x609 0x00200000  ;# set DMA Mem_Size
}

proc Stop_Electronics {} {

     WriteAIDARegister 0x300 0   ;#    unset enable ASIC readout
     WriteAIDARegister 0x310 0   ;#    unset enable MBS
     WriteAIDARegister 0x500 0   ;#    unset enable DISCRIMINATOR readout

     WriteAIDARegister 0x600 0        ;#  clear DMA
}

proc Go_Electronics {} {

    set Me $::DataAcquisition::ACQSERVER
    set TSMaster $::AIDA::TSMaster

     WriteAIDARegister 0x300 0x4000   ;#  set RDO_reset
     WriteAIDARegister 0x300 0x0000   ;#  unset RDO_reset

     WriteAIDARegister 0x600 0x8000   ;#  set reset DMA
     WriteAIDARegister 0x600 0        ;#  and clear

     puts "Go_Electronics: DMA State Machine [format 0x%04X [ReadAIDARegister 0x601]]"

#     WriteAIDARegister 0x60a 0		; # set to data not test counter
#     WriteAIDARegister 0x308 0		; # set to data not test counter

     after 1000			 ; # Pause 1 second to allow the readout to settle

     WriteAIDARegister 0x600 1        ;#  enable DMA
     WriteAIDARegister 0x310 1        ;#  enable MBS

     WriteAIDARegister 0x300 0x0001   ;#  set enable ASIC readout
     WriteAIDARegister 0x500 0x0001   ;#  set enable discriminators

}

    proc TerminateExec {} {

        unmap_spectra
        DataAcqCommand 22 "Terminating DataAcq program" 0 2

    }

    proc LoadExec {} {

       set z [catch {eval exec nice [file join /MIDAS Data_Acq bin linux-ppc_4xx AidaExecV4] &} m]
       if {$z != 0} {puts "RunControl LoadExec returned z=$z and $m"}

    }

    make-UIregisters

    Set_Options




